@@ -39,74 +39,7 @@ static garray_T	user_digraphs = {0, 0, (int)sizeof(digr_T), 10, NULL};
  * compilers cannot handle them (Amiga SAS/C is the most picky one).
  */
 static digr_T digraphdefault[] =
-#if defined(MSDOS)
-	/*
-	 * MSDOS digraphs.
-	 */
-       {{'C', ',', 128},	/* ~@ XX */
-	{'u', '"', 129},	/*  */
-	{'e', '\'', 130},	/*  */
-	{'a', '^', 131},	/*  */
-	{'a', '"', 132},	/*  */
-	{'a', '`', 133},	/*  */
-	{'a', '@', 134},	/*  */
-	{'c', ',', 135},	/* ~G XX */
-	{'e', '^', 136},	/* ~H XX */
-	{'e', '"', 137},	/*  */
-	{'e', '`', 138},	/*  */
-	{'i', '"', 139},	/*  */
-	{'i', '^', 140},	/*  */
-	{'i', '`', 141},	/*  */
-	{'A', '"', 142},	/* ~N XX */
-	{'A', '@', 143},	/*  */
-	{'E', '\'', 144},	/*  */
-	{'a', 'e', 145},	/*  */
-	{'A', 'E', 146},	/*  */
-	{'o', '^', 147},	/*  */
-	{'o', '"', 148},	/*  */
-	{'o', '`', 149},	/*  */
-	{'u', '^', 150},	/*  */
-	{'u', '`', 151},	/*  */
-	{'y', '"', 152},	/*  */
-	{'O', '"', 153},	/*  */
-	{'U', '"', 154},	/*  */
-	{'c', '|', 155},	/*  */
-	{'$', '$', 156},	/*  */
-	{'Y', '-', 157},	/* ~] XX */
-	{'P', 't', 158},	/*  */
-	{'f', 'f', 159},	/*  */
-	{'a', '\'', 160},	/*  */
-	{'i', '\'', 161},	/*  */
-	{'o', '\'', 162},	/*  */
-	{'u', '\'', 163},	/* x XX */
-	{'n', '~', 164},	/*  */
-	{'N', '~', 165},	/*  */
-	{'a', 'a', 166},	/*  */
-	{'o', 'o', 167},	/*  */
-	{'~', '?', 168},	/*  */
-	{'-', 'a', 169},	/*  */
-	{'a', '-', 170},	/*  */
-	{'1', '2', 171},	/*  */
-	{'1', '4', 172},	/*  */
-	{'~', '!', 173},	/*  */
-	{'<', '<', 174},	/*  */
-	{'>', '>', 175},	/*  */
-
-	{'s', 's', 225},	/*  */
-	{'j', 'u', 230},	/*  */
-	{'o', '/', 237},	/*  */
-	{'+', '-', 241},	/*  */
-	{'>', '=', 242},	/*  */
-	{'<', '=', 243},	/*  */
-	{':', '-', 246},	/*  */
-	{'~', '~', 247},	/*  */
-	{'~', 'o', 248},	/*  */
-	{'2', '2', 253},	/*  */
-	{NUL, NUL, NUL}
-	};
-
-#else	/* !MSDOS */
-# ifdef __MINT__
+#ifdef __MINT__
 
 	/*
 	 * ATARI digraphs
@@ -171,8 +104,8 @@ static digr_T digraphdefault[] =
 	{NUL, NUL, NUL}
 	};
 
-# else	/* !__MINT__ */
-#  ifdef HPUX_DIGRAPHS
+#else	/* !__MINT__ */
+# ifdef HPUX_DIGRAPHS
 
 	/*
 	 * different HPUX digraphs
@@ -275,9 +208,9 @@ static digr_T digraphdefault[] =
 	{NUL, NUL, NUL}
 	};
 
-#  else	/* !HPUX_DIGRAPHS */
+# else	/* !HPUX_DIGRAPHS */
 
-#   ifdef EBCDIC
+#  ifdef EBCDIC
 
 	/*
 	 * EBCDIC - ISO digraphs
@@ -387,8 +320,8 @@ static digr_T digraphdefault[] =
 	{NUL, NUL, NUL}
 	};
 
-#   else
-#    if defined(MACOS) && !defined(FEAT_MBYTE)
+#  else
+#   if defined(MACOS) && !defined(FEAT_MBYTE)
 
 	/*
 	 * Macintosh digraphs
@@ -516,9 +449,9 @@ static digr_T digraphdefault[] =
 	{NUL, NUL, NUL}
 	};
 
-#    else	/* !MACOS */
+#   else	/* !MACOS */
 
-#     ifdef OLD_DIGRAPHS
+#    ifdef OLD_DIGRAPHS
 
 	/*
 	 * digraphs compatible with Vim 5.x
@@ -625,7 +558,7 @@ static digr_T digraphdefault[] =
 	{'y', '"', 255},	/* x XX */
 	{NUL, NUL, NUL}
 	};
-#     else /* OLD_DIGRAPHS */
+#    else /* OLD_DIGRAPHS */
 
 	/*
 	 * digraphs for Unicode from RFC1345
@@ -2001,13 +1934,12 @@ static digr_T digraphdefault[] =
 	{NUL, NUL, NUL}
        };
 
-#     endif /* OLD_DIGRAPHS */
+#    endif /* OLD_DIGRAPHS */
 
-#    endif /* Macintosh */
-#   endif /* EBCDIC */
-#  endif    /* !HPUX_DIGRAPHS */
-# endif	/* !__MINT__ */
-#endif	/* !MSDOS */
+#   endif /* Macintosh */
+#  endif /* EBCDIC */
+# endif    /* !HPUX_DIGRAPHS */
+#endif	/* !__MINT__ */
 
 /*
  * handle digraphs after typing a character
